# vim:set encoding=utf-8:

# Copyright:
#   © 2009 Sanghyeon Seo <sanxiyn@gmail.com>
#
# License:
#   Public Domain

# @see http://confluence.atlassian.com/display/JIRA/JIRA+RPC+Services

# JIRA seems to require a user account to use its SOAP service.
# Therefore, you need to create a user with ID bts-link, PW bts-link
# before adding an entry to btslink.cfg. Sorry, this can't be automated.
#
# ST, 2009-11-05: ^^ really? apache seems not to need this.

import SOAPpy

from __init__ import *

class JIRAData:
    def __init__(self, bts, id):
        issue = bts.soap.getIssue(bts.auth, id)

        self.id = id or failwith(uri, 'JIRA: no id')
        self.status = bts.status_map[issue.status]
        self.resolution = None

        if issue.resolution:
            self.resolution = bts.resolution_map[issue.resolution]
            if issue.resolution == 'Duplicate':
                raise DupeExn(uri)

class RemoteJIRA(RemoteBts):
    def __init__(self, cnf):
        bugre = r'^%(uri)s/browse/([A-Z]+-[0-9]+)$'
        urifmt = '%(uri)s/browse/%(id)s'
        RemoteBts.__init__(self, cnf, bugre, urifmt, JIRAData)

        uri = cnf['uri']
        wsdl = uri + '/rpc/soap/jirasoapservice-v2?wsdl'
        try:
            self.soap = SOAPpy.WSDL.Proxy(wsdl)
        except:
            print "E: error accessing", wsdl
            return None

        try:
            self.auth = self.soap.login('bts-link', 'bts-link')
        except:
            # mainly for puppet
            self.auth = self.soap.login('bts-link', 'mQJ1lEYaHcsoU')

        self.status_map = {}
        for status in self.soap.getStatuses(self.auth):
            self.status_map[status.id] = status.name

        self.resolution_map = {}
        for resolution in self.soap.getResolutions(self.auth):
            self.resolution_map[resolution.id] = resolution.name

    def _getReportData(self, uri):
        id = self.extractBugid(uri)
        if not id: return None

        data = JIRAData(self, id)
        return data

    def isClosing(self, status, resolution):
        return status in ('Resolved', 'Closed')

    def isWontfix(self, status, resolution):
        return resolution in ("Won't Fix",)

RemoteJIRA.register('jira', RemoteJIRA)
